@@ -9,8 +9,8 @@ from skbio.io import read as read_sequence
 from skbio.io import write as write_sequence
 
 from mag_annotator.database_handler import DatabaseHandler
-from mag_annotator.annotate_bins import filter_db_locs, annotate_fastas
-from mag_annotator.utils import get_database_locs, get_ids_from_annotation
+from mag_annotator.annotate_bins import annotate_fastas
+from mag_annotator.utils import get_ids_from_annotation
 
 VMAG_DBS_TO_ANNOTATE = ('kegg', 'kofam', 'kofam_ko_list', 'uniref', 'peptidase', 'pfam', 'dbcan', 'viral', 'vogdb')
 VIRSORTER_COLUMN_NAMES = ['gene_name', 'start_position', 'end_position', 'length', 'strandedness',
@@ -43,21 +43,56 @@ VIRAL_PEPTIDASES_MEROPS = {'A02H', 'A02G', 'A02F', 'A02E', 'A02D', 'A02C', 'A02B
                            'T01B', 'T01A', 'T03', 'U32', 'U40'}
 
 
+def is_affi_tab_not_fasta(input_file: str) -> bool:
+    """
+    Checks that a file is a VIRSorter affi tab file.
+
+    :param input_file:
+        the input file that the user provided.
+    :returns:
+        True if there are 11 delimiters on the second line.
+    """
+    with open(input_file) as input_io:
+        num_delimiter = input_io.readlines()[1].count('|')
+    if num_delimiter == 11:
+        return True
+    return False
+
+
 def remove_bad_chars_fasta(fasta):
+    if is_affi_tab_not_fasta(fasta):
+        raise ValueError("The input file format matches virsorter affi contigs not fasta, please check "
+                         "that the flags match the file type: '-v' for virsorter, and '-i' for fasta.")
+
     new_headers = list()
     new_seqs = list()
     for seq in read_sequence(fasta, format='fasta'):
         new_header = seq.metadata['id']
         new_header = new_header.replace(';', '_').replace('=', '_')
         if new_header in new_headers:
-            raise ValueError('Replacement of ; or = with _ generated reduntant sequence headers, must be modified '
+            raise ValueError('Replacement of ; or = with _ generated redundant sequence headers, must be modified '
                              'manually')
         seq.metadata['id'] = new_header
         new_seqs.append(seq)
     return new_seqs
 
 
-def remove_bad_chars_virsorter_affi_contigs(virsorter_in):
+def remove_bad_chars_virsorter_affi_contigs(virsorter_in: str) -> str:
+    """
+    Determine and call the appropriate function to cleans incompatible characters from VIRSorter
+    or VIRSorter2 affi headers.
+
+    :param virsorter_in:
+        Path to a affi tab file provide for DRAM by VIRSorter or VIRSorter2.
+    :returns:
+        String of file contents with headers cleaned.
+    :raises ValueError:
+        If the input format does not match VIRSorter or VIRSorter2.
+    """
+    if not is_affi_tab_not_fasta(virsorter_in):
+        raise ValueError("The input file format does not match virsorter affi contigs, please check "
+                         "that the flags match the file type: '-v' for virsorter, and '-i' for "
+                         "fasta.")
     new_lines = list()
     for line in open(virsorter_in).readlines():
         line = line.strip()
@@ -73,7 +108,21 @@ def remove_bad_chars_virsorter_affi_contigs(virsorter_in):
     return '%s\n' % '\n'.join(new_lines)
 
 
-def remove_bad_chars(input_fasta=None, input_virsorter_affi_contigs=None, output=None):
+def remove_bad_chars(input_fasta: str = None, input_virsorter_affi_contigs: str = None, output: str = None):
+    """
+    Determine and call the function to cleans incompatible characters from headers of virsorter
+    affi or fasta file.
+
+    :param input_fasta:
+        Path to a fasta file provide for DRAM by VIRSorter or VIRSorter2
+    :param input_virsorter_affi_contigs:
+        Path to a affi tab file provide for DRAM by VIRSorter
+        or VIRSorter2
+    :param output:
+        Desired path for the new file clean of incompatible characters
+    :raises ValueError:
+        Check format, and invalid arguments.
+    """
     if input_fasta is None and input_virsorter_affi_contigs is None:
         raise ValueError('Must provide either input fasta or virsorter affi contigs')
     elif input_fasta is not None:
@@ -242,7 +291,7 @@ def get_metabolic_flags(annotations, metabolic_genes, amgs, verified_amgs, scaff
             flags = ''
             gene_annotations = set(get_ids_from_annotation(pd.DataFrame(row).transpose()).keys())  # TODO: Fix
             # is viral
-            if not pd.isna(row['vogdb_categories']):
+            if 'vogdb_categories' in row.index and not pd.isna(row['vogdb_categories']):
                 if len({'Xr', 'Xs'} & set(row['vogdb_categories'].split(';'))) > 0:
                     flags += 'V'
             # is metabolic
@@ -298,7 +347,12 @@ def get_amg_ids(amg_frame):
     return ko_amgs | ec_amgs | pfam_amgs
 
 
-def get_virsorter_affi_contigs_name(scaffold):
+def get_virsorter2_affi_contigs_name(scaffold):
+    virsorter_scaffold_name = scaffold.replace('|', '_')
+    return virsorter_scaffold_name
+
+
+def get_virsorter_original_affi_contigs_name(scaffold):
     prophage_match = re.search(r'_gene_\d*_gene_\d*-\d*-\d*-cat_[123456]$', scaffold)
     plain_match = re.search(r'-cat_[123456]$', scaffold)
     if prophage_match is not None:
@@ -306,14 +360,26 @@ def get_virsorter_affi_contigs_name(scaffold):
     elif plain_match is not None:
         virsorter_scaffold_name = scaffold[:plain_match.start()]
     else:
-        raise ValueError("Can't find VIRSorter endings on fasta header: %s" % scaffold)
+        raise ValueError("Format is not VIRSorter2, and can't find VIRSorter "
+                         "endings on fasta header: %s" % scaffold)
     return virsorter_scaffold_name
 
 
-def add_dramv_scores_and_flags(annotations, db_locs=None, virsorter_hits=None, input_fasta=None):
+def get_virsorter_affi_contigs_name(scaffold):
+    """
+    :param scaffold: A raw string from the fasta header
+    :returns: A string returned from the appropriate parser
+    """
+    virsorter2_match = re.search(r'\|\|', scaffold)
+    if virsorter2_match is not None:
+        return get_virsorter2_affi_contigs_name(scaffold)
+    return get_virsorter_original_affi_contigs_name(scaffold)
+
+
+def add_dramv_scores_and_flags(annotations, db_handler, virsorter_hits=None, input_fasta=None):
     # setting up scoring viral genes
-    amg_database_frame = pd.read_csv(db_locs['amg_database'], sep='\t')
-    genome_summary_form = pd.read_csv(db_locs['genome_summary_form'], sep='\t', index_col=0)
+    amg_database_frame = pd.read_csv(db_handler.dram_sheet_locs['amg_database'], sep='\t')
+    genome_summary_form = pd.read_csv(db_handler.dram_sheet_locs['genome_summary_form'], sep='\t', index_col=0)
     genome_summary_form = genome_summary_form.loc[genome_summary_form.potential_amg]
 
     # add auxiliary score
@@ -352,10 +418,11 @@ def add_dramv_scores_and_flags(annotations, db_locs=None, virsorter_hits=None, i
     return annotations
 
 
-def annotate_vgfs(input_fasta, virsorter_affi_contigs=None, output_dir='.', min_contig_size=2500, prodigal_mode='meta',
-                  trans_table='11', bit_score_threshold=60, rbh_bit_score_threshold=350, custom_db_name=(),
-                  custom_fasta_loc=(), use_uniref=False, low_mem_mode=False, skip_trnascan=False, keep_tmp_dir=True,
-                  threads=10, verbose=True):
+def annotate_vgfs(input_fasta, virsorter_affi_contigs=None, output_dir='.', min_contig_size=2500, split_contigs=False,
+                  prodigal_mode='meta', trans_table='11', bit_score_threshold=60, rbh_bit_score_threshold=350,
+                  custom_db_name=(), custom_fasta_loc=(), custom_hmm_loc=(), custom_hmm_cutoffs_loc=(),
+                  custom_hmm_name=(), use_uniref=False, kofam_use_dbcan2_thresholds=False, skip_trnascan=False,
+                  keep_tmp_dir=True, low_mem_mode=False, threads=10, verbose=True):
     # set up
     start_time = datetime.now()
     print('%s: Viral annotation started' % str(datetime.now()))
@@ -370,45 +437,49 @@ def annotate_vgfs(input_fasta, virsorter_affi_contigs=None, output_dir='.', min_
                       'training to work well.')
 
     # get database locations
-    db_locs = get_database_locs()
-    db_handler = DatabaseHandler(db_locs['description_db'])
-    db_locs_anno = filter_db_locs(db_locs, low_mem_mode, use_uniref, VMAG_DBS_TO_ANNOTATE)
+    db_handler = DatabaseHandler()
+    db_handler.filter_db_locs(low_mem_mode, use_uniref, True, VMAG_DBS_TO_ANNOTATE)
 
     if virsorter_affi_contigs is not None:
         virsorter_hits = get_virsorter_hits(virsorter_affi_contigs)
     else:
         virsorter_hits = None
 
-    # split sequences into seperate fastas
     mkdir(output_dir)
-    contig_dir = path.join(output_dir, 'vMAGs')
-    mkdir(contig_dir)
-    contig_locs = list()
-    for seq in read_sequence(input_fasta, format='fasta'):
-        if len(seq) >= min_contig_size:
-            if '=' in seq.metadata['id'] or ';' in seq.metadata['id']:
-                raise ValueError('FASTA headers must not have = or ; before the first space (%s). To run DRAM-v you '
-                                 'must rerun VIRSorter with = and ; removed from the headers or run DRAM-v.py '
-                                 'remove_bad_characters and then rerun DRAM-v' % seq.metadata['id'])
-            if virsorter_hits is not None:
-                if get_virsorter_affi_contigs_name(seq.metadata['id']) not in virsorter_hits['name'].values:
-                    raise ValueError("No virsorter calls found in %s for scaffold %s from input fasta" %
-                                     (virsorter_affi_contigs, seq.metadata['id']))
-            contig_loc = path.join(contig_dir, '%s.fasta' % seq.metadata['id'])
-            write_sequence((i for i in [seq]), format='fasta', into=contig_loc)
-            contig_locs.append(contig_loc)
+    if split_contigs:
+        # split sequences into separate fastas
+        contig_dir = path.join(output_dir, 'vMAGs')
+        mkdir(contig_dir)
+
+        contig_locs = list()
+        for seq in read_sequence(input_fasta, format='fasta'):
+            if len(seq) >= min_contig_size:
+                if '=' in seq.metadata['id'] or ';' in seq.metadata['id']:
+                    raise ValueError('FASTA headers must not have = or ; before the first space (%s). To run DRAM-v '
+                                     'you must rerun VIRSorter with = and ; removed from the headers or run DRAM-v.py '
+                                     'remove_bad_characters and then rerun DRAM-v' % seq.metadata['id'])
+                if virsorter_hits is not None:
+                    if get_virsorter_affi_contigs_name(seq.metadata['id']) not in virsorter_hits['name'].values:
+                        raise ValueError("No virsorter calls found in %s for scaffold %s from input fasta" %
+                                         (virsorter_affi_contigs, seq.metadata['id']))
+                contig_loc = path.join(contig_dir, '%s.fasta' % seq.metadata['id'])
+                write_sequence((i for i in [seq]), format='fasta', into=contig_loc)
+                contig_locs.append(contig_loc)
+    else:
+        contig_locs = [input_fasta]
 
     # annotate vMAGs
     rename_bins = False
-    annotations = annotate_fastas(contig_locs, output_dir, db_locs_anno, db_handler, min_contig_size, prodigal_mode,
-                                  trans_table, bit_score_threshold, rbh_bit_score_threshold, custom_db_name,
-                                  custom_fasta_loc, skip_trnascan, rename_bins, keep_tmp_dir, start_time, threads,
-                                  verbose)
+    annotations = annotate_fastas(contig_locs, output_dir, db_handler, min_contig_size, prodigal_mode, trans_table,
+                                  bit_score_threshold, rbh_bit_score_threshold, custom_db_name, custom_fasta_loc,
+                                  custom_hmm_name, custom_hmm_loc, custom_hmm_cutoffs_loc, kofam_use_dbcan2_thresholds,
+                                  skip_trnascan, rename_bins, keep_tmp_dir, start_time, threads, verbose)
     print('%s: Annotations complete, assigning auxiliary scores and flags' % str(datetime.now() - start_time))
 
-    annotations = add_dramv_scores_and_flags(annotations, db_locs, virsorter_hits, input_fasta)
+    annotations = add_dramv_scores_and_flags(annotations, db_handler, virsorter_hits, input_fasta)
 
     # write annotations
     annotations.to_csv(path.join(output_dir, 'annotations.tsv'), sep='\t')
 
     print("%s: Completed annotations" % str(datetime.now() - start_time))
+
