@@ -1,21 +1,19 @@
-from os import path, mkdir, remove
-from shutil import copy2
+from os import path, mkdir
 from datetime import datetime
 from shutil import move, rmtree
 from glob import glob
-import json
 import gzip
 import tarfile
-import pandas as pd
 from collections import defaultdict
 from skbio import read as read_sequence
 from skbio import write as write_sequence
 
-from mag_annotator.utils import run_process, make_mmseqs_db, get_database_locs, download_file, merge_files, \
-    get_config_loc, remove_prefix
 from mag_annotator.database_handler import DatabaseHandler
-from mag_annotator.database_setup import create_description_db
+from mag_annotator.utils import run_process, make_mmseqs_db, download_file, merge_files, remove_prefix
 
+DEFAULT_DBCAN_RELEASE = '10'
+DEFAULT_DBCAN_DATE = '07292021'
+DEFAULT_UNIREF_VERSION = '90'
 
 # TODO: check if dbcan or pfam is down, raise appropriate error
 # TODO: upgrade to pigz?
@@ -25,22 +23,6 @@ def get_iso_date():
     return datetime.today().strftime('%Y%m%d')
 
 
-def check_file_exists(db_loc):
-    if db_loc is None:
-        return True
-    elif path.isfile(db_loc):
-        return True
-    else:
-        raise ValueError("Database location does not exist: %s" % db_loc)
-
-
-def make_header_dict_from_mmseqs_db(mmseqs_db):
-    mmseqs_headers_handle = open('%s_h' % mmseqs_db, 'rb')
-    mmseqs_headers = mmseqs_headers_handle.read().decode(errors='ignore')
-    mmseqs_headers = [i.strip() for i in mmseqs_headers.strip().split('\n\x00') if len(i) > 0]
-    return [{'id': i.split(' ')[0], 'description': i} for i in mmseqs_headers]
-
-
 def generate_modified_kegg_fasta(kegg_fasta, gene_ko_link_loc=None):
     """Takes kegg fasta file and gene ko link file, adds kos not already in headers to headers"""
     genes_ko_dict = defaultdict(list)
@@ -100,12 +82,12 @@ def download_and_process_kofam_ko_list(kofam_ko_list_gz=None, output_dir='.', ve
     return kofam_ko_list
 
 
-def download_and_process_uniref(uniref_fasta_zipped=None, output_dir='.', uniref_version='90', threads=10,
+def download_and_process_uniref(uniref_fasta_zipped=None, output_dir='.', uniref_version=DEFAULT_UNIREF_VERSION, threads=10,
                                 verbose=True):
     """"""
     if uniref_fasta_zipped is None:  # download database if not provided
         uniref_fasta_zipped = path.join(output_dir, 'uniref%s.fasta.gz' % uniref_version)
-        uniref_url = 'ftp://ftp.uniprot.org/pub/databases/uniprot/uniref/uniref%s/uniref%s.fasta.gz' % \
+        uniref_url = 'https://ftp.uniprot.org/pub/databases/uniprot/uniref/uniref%s/uniref%s.fasta.gz' % \
                      (uniref_version, uniref_version)
         download_file(uniref_url, uniref_fasta_zipped, verbose=verbose)
     uniref_mmseqs_db = path.join(output_dir, 'uniref%s.%s.mmsdb' % (uniref_version, get_iso_date()))
@@ -141,64 +123,31 @@ def download_pfam_descriptions(output_dir='.', verbose=True):
     return pfam_hmm_dat
 
 
-def process_pfam_descriptions(pfam_hmm_dat):
-    if pfam_hmm_dat.endswith('.gz'):
-        f = gzip.open(pfam_hmm_dat, 'r').read().decode('utf-8')
+def download_dbcan(dbcan_hmm=None, output_dir='.', dbcan_release=DEFAULT_DBCAN_RELEASE, verbose=True):
+    dbcan_hmm = path.join(output_dir, f"dbCAN-HMMdb-V{dbcan_release}.txt" )
+    if int(dbcan_release) < int(DEFAULT_DBCAN_RELEASE):
+        link_path = f"http://bcb.unl.edu/dbCAN2/download/Databases/dbCAN-HMMdb-V{dbcan_release}.txt"
     else:
-        f = open(pfam_hmm_dat).read()
-    entries = f.strip().split('//')
-    description_list = list()
-    for i, entry in enumerate(entries):
-        if len(entry) > 0:
-            entry = entry.split('\n')
-            ascession = None
-            description = None
-            for line in entry:
-                line = line.strip()
-                if line.startswith('#=GF AC'):
-                    ascession = line.split('   ')[-1]
-                if line.startswith('#=GF DE'):
-                    description = line.split('   ')[-1]
-            description_list.append({'id': ascession, 'description': description})
-    return description_list
+        link_path = f"http://bcb.unl.edu/dbCAN2/download/dbCAN-HMMdb-V{dbcan_release}.txt"
+
+    print(f"Downloading dbCAN from: {link_path}")
+    download_file(link_path, dbcan_hmm, verbose=verbose)
+    return dbcan_hmm
 
 
-def download_and_process_dbcan(dbcan_hmm=None, output_dir='.', dbcan_release='9', verbose=True):
-    if dbcan_hmm is None:  # download database if not provided
-        dbcan_hmm = path.join(output_dir, 'dbCAN-HMMdb-V%s.txt' % dbcan_release)
-        if int(dbcan_release) < 9:
-            download_file('http://bcb.unl.edu/dbCAN2/download/Databases/dbCAN-HMMdb-V%s.txt' % dbcan_release, dbcan_hmm,
-                          verbose=verbose)
-        else:
-            download_file('http://bcb.unl.edu/dbCAN2/download/dbCAN-HMMdb-V%s.txt' % dbcan_release, dbcan_hmm,
-                          verbose=verbose)
+def process_dbcan(dbcan_hmm, verbose=True):
     run_process(['hmmpress', '-f', dbcan_hmm], verbose=verbose)
     return dbcan_hmm
 
 
-def download_dbcan_descriptions(output_dir='.', upload_date='07302020', verbose=True):
+def download_dbcan_descriptions(output_dir='.', dbcan_release=DEFAULT_DBCAN_RELEASE, upload_date=DEFAULT_DBCAN_DATE, verbose=True):
     dbcan_fam_activities = path.join(output_dir, 'CAZyDB.%s.fam-activities.txt' % upload_date)
-    download_file('http://bcb.unl.edu/dbCAN2/download/Databases/CAZyDB.%s.fam-activities.txt' % upload_date,
-                  dbcan_fam_activities, verbose=verbose)
+    link_path = f"https://bcb.unl.edu/dbCAN2/download/Databases/V{dbcan_release}/CAZyDB.{upload_date}.fam-activities.txt"
+    print(f"Downloading dbCAN family activities from : {link_path}")
+    download_file(link_path, dbcan_fam_activities, verbose=verbose)
     return dbcan_fam_activities
 
 
-def process_dbcan_descriptions(dbcan_fam_activities):
-    f = open(dbcan_fam_activities)
-    description_list = list()
-    for line in f.readlines():
-        if not line.startswith('#') and len(line.strip()) != 0:
-            line = line.strip().split()
-            if len(line) == 1:
-                description = line[0]
-            elif line[0] == line[1]:
-                description = ' '.join(line[1:])
-            else:
-                description = ' '.join(line)
-            description_list.append({'id': line[0], 'description': description.replace('\n', ' ')})
-    return description_list
-
-
 def download_and_process_viral_refseq(merged_viral_faas=None, output_dir='.', viral_files=2, threads=10, verbose=True):
     """Can only download newest version"""
     # download all of the viral protein files, need to know the number of files
@@ -255,14 +204,6 @@ def download_vog_annotations(output_dir, vogdb_version='latest', verbose=True):
     return vog_annotations
 
 
-def process_vogdb_descriptions(vog_annotations):
-    annotations_table = pd.read_csv(vog_annotations, sep='\t', index_col=0)
-    annotations_list = [{'id': vog, 'description': '%s; %s' % (row['ConsensusFunctionalDescription'],
-                                                               row['FunctionalCategory'])}
-                        for vog, row in annotations_table.iterrows()]
-    return annotations_list
-
-
 def download_and_process_genome_summary_form(output_dir, branch='master', verbose=True):
     genome_summary_form = path.join(output_dir, 'genome_summary_form.%s.tsv' % get_iso_date())
     download_file('https://raw.githubusercontent.com/shafferm/DRAM/%s/data/genome_summary_form.tsv' % branch,
@@ -298,119 +239,18 @@ def download_and_process_amg_database(output_dir, branch='master', verbose=True)
     return amg_database
 
 
-def check_exists_and_add_to_location_dict(loc, name, dict_to_update):
-    if loc is not None:  # if location give and exists then add to dict, else raise ValueError
-        if check_file_exists(loc):
-            dict_to_update[name] = path.realpath(loc)
-    else:  # if location not given and is not in dict then set to none, else leave previous value
-        if name not in dict_to_update:
-            dict_to_update[name] = None
-    return dict_to_update
-
-
-def add_to_description_db(loc, name, get_description_list, db_handler):
-    if loc is not None:  # if location give and exists then add to dict, else raise ValueError
-        description_list = get_description_list(loc)
-        db_handler.add_descriptions_to_database(description_list, name, clear_table=True)
-
-
-def set_database_paths(kegg_db_loc=None, kofam_hmm_loc=None, kofam_ko_list_loc=None, uniref_db_loc=None,
-                       pfam_db_loc=None, pfam_hmm_dat=None, dbcan_db_loc=None, dbcan_fam_activities=None,
-                       viral_db_loc=None, peptidase_db_loc=None, vogdb_db_loc=None, vog_annotations=None,
-                       description_db_loc=None, genome_summary_form_loc=None, module_step_form_loc=None,
-                       etc_module_database_loc=None, function_heatmap_form_loc=None, amg_database_loc=None,
-                       start_time=None, config_loc=None, use_current_locs=True, update_description_db=False):
-    if start_time is None:
-        start_time = datetime.now()
-    print('%s: Setting database paths' % str(datetime.now() - start_time))
-    if use_current_locs:
-        db_dict = get_database_locs()
+def check_file_exists(db_loc):
+    if db_loc is None:
+        return True
+    elif path.isfile(db_loc):
+        return True
     else:
-        db_dict = {}
-
-    db_dict = check_exists_and_add_to_location_dict(kegg_db_loc, 'kegg', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(kofam_hmm_loc, 'kofam', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(kofam_ko_list_loc, 'kofam_ko_list', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(uniref_db_loc, 'uniref', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(pfam_db_loc, 'pfam', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(pfam_hmm_dat, 'pfam_hmm_dat', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(dbcan_db_loc, 'dbcan', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(dbcan_fam_activities, 'dbcan_fam_activities', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(viral_db_loc, 'viral', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(peptidase_db_loc, 'peptidase', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(vogdb_db_loc, 'vogdb', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(vog_annotations, 'vog_annotations', db_dict)
-
-    db_dict = check_exists_and_add_to_location_dict(genome_summary_form_loc, 'genome_summary_form', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(module_step_form_loc, 'module_step_form', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(etc_module_database_loc, 'etc_module_database', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(function_heatmap_form_loc, 'function_heatmap_form', db_dict)
-    db_dict = check_exists_and_add_to_location_dict(amg_database_loc, 'amg_database', db_dict)
-    print('%s: Database locations added to CONFIG' % str(datetime.now() - start_time))
-
-    if update_description_db:
-        if description_db_loc is None:
-            description_db_loc = db_dict['description_db']
-        populate_description_db(description_db_loc, db_dict, start_time)
-        print('%s: Database descriptions updated' % str(datetime.now() - start_time))
-    db_dict = check_exists_and_add_to_location_dict(description_db_loc, 'description_db', db_dict)
-
-    # change data paths
-    if config_loc is None:
-        config_loc = get_config_loc()
-    with open(config_loc, 'w') as f:
-        f.write(json.dumps(db_dict))
-    print('%s: Database locations set' % str(datetime.now() - start_time))
-
-
-def populate_description_db(output_loc=None, db_dict=None, start_time=None):
-    if start_time is None:
-        start_time = datetime.now()
-        print('%s: Populating description database' % str(datetime.now() - start_time))
-    # setup
-    if db_dict is None:
-        db_dict = get_database_locs()
-
-    if db_dict.get('description_db') is None and output_loc is not None:
-        db_dict['description_db'] = output_loc
-    elif db_dict.get('description_db') is None and output_loc is None:
-        raise ValueError('Must provide output location if description db location is not set in configuration')
-    elif path.exists(db_dict['description_db']):
-        remove(db_dict['description_db'])
-
-    create_description_db(db_dict['description_db'])
-    db_handler = DatabaseHandler(db_dict['description_db'])
-    print('%s: Database connection established' % str(datetime.now() - start_time))
-
-    # fill database
-    add_to_description_db(db_dict['kegg'], 'kegg_description', make_header_dict_from_mmseqs_db,
-                          db_handler)
-    print('%s: KEGG descriptions added to description database' % str(datetime.now() - start_time))
-    add_to_description_db(db_dict['uniref'], 'uniref_description', make_header_dict_from_mmseqs_db,
-                          db_handler)
-    print('%s: UniRef descriptions added to description database' % str(datetime.now() - start_time))
-    add_to_description_db(db_dict['pfam_hmm_dat'], 'pfam_description', process_pfam_descriptions,
-                          db_handler)
-    print('%s: PFAM descriptions added to description database' % str(datetime.now() - start_time))
-    add_to_description_db(db_dict['dbcan_fam_activities'], 'dbcan_description',
-                          process_dbcan_descriptions, db_handler)
-    print('%s: dbCAN descriptions added to description database' % str(datetime.now() - start_time))
-    add_to_description_db(db_dict['viral'], 'viral_description', make_header_dict_from_mmseqs_db,
-                          db_handler)
-    print('%s: RefSeq viral descriptions added to description database' % str(datetime.now() - start_time))
-    add_to_description_db(db_dict['peptidase'], 'peptidase_description',
-                          make_header_dict_from_mmseqs_db, db_handler)
-    print('%s: MEROPS descriptions added to description database' % str(datetime.now() - start_time))
-    add_to_description_db(db_dict['vog_annotations'], 'vogdb_description', process_vogdb_descriptions,
-                          db_handler)
-    print('%s: VOGdb descriptions added to description database' % str(datetime.now() - start_time))
-    print('%s: Description database populated' % str(datetime.now() - start_time))
-    # TODO: if output loc is given then set in config file where the description database is
+        raise ValueError("Database location does not exist: %s" % db_loc)
 
 
 def prepare_databases(output_dir, kegg_loc=None, gene_ko_link_loc=None, kofam_hmm_loc=None, kofam_ko_list_loc=None,
-                      kegg_download_date=None, uniref_loc=None, uniref_version='90', pfam_loc=None, pfam_hmm_dat=None,
-                      dbcan_loc=None, dbcan_version='9', dbcan_fam_activities=None, dbcan_date='07302020',
+                      kegg_download_date=None, uniref_loc=None, uniref_version=DEFAULT_UNIREF_VERSION, pfam_loc=None, pfam_hmm_dat=None,
+                      dbcan_loc=None, dbcan_version=DEFAULT_DBCAN_RELEASE, dbcan_fam_activities=None, dbcan_date=DEFAULT_DBCAN_DATE,
                       viral_loc=None, peptidase_loc=None, vogdb_loc=None, vogdb_version='latest', vog_annotations=None,
                       genome_summary_form_loc=None, module_step_form_loc=None, etc_module_database_loc=None,
                       function_heatmap_form_loc=None, amg_database_loc=None, skip_uniref=False,
@@ -447,8 +287,22 @@ def prepare_databases(output_dir, kegg_loc=None, gene_ko_link_loc=None, kofam_hm
     temporary = path.join(output_dir, 'database_files')
     mkdir(temporary)
 
-    # get databases
+    # Download DBs
+    if dbcan_fam_activities is None:
+        dbcan_fam_activities = download_dbcan_descriptions(
+            output_dir=output_dir, dbcan_release=dbcan_version,
+            upload_date=dbcan_date, verbose=verbose)
+    if pfam_hmm_dat is None:
+        pfam_hmm_dat = download_pfam_descriptions(output_dir, verbose=verbose)
+    if dbcan_loc is None:
+        dbcan_loc = download_dbcan(temporary, dbcan_release=dbcan_version, verbose=verbose)
+
+    # Process databases
     output_dbs = dict()
+
+    output_dbs['dbcan_db_loc'] = process_dbcan(dbcan_loc, verbose=verbose)
+    print('%s: dbCAN database processed' % str(datetime.now() - start_time))
+
     if kegg_loc is not None:
         output_dbs['kegg_db_loc'] = process_kegg_db(temporary, kegg_loc, gene_ko_link_loc, kegg_download_date, threads,
                                                     verbose)
@@ -460,9 +314,6 @@ def prepare_databases(output_dir, kegg_loc=None, gene_ko_link_loc=None, kofam_hm
     output_dbs['pfam_db_loc'] = download_and_process_pfam(pfam_loc, temporary,
                                                           threads=threads, verbose=verbose)
     print('%s: PFAM database processed' % str(datetime.now() - start_time))
-    output_dbs['dbcan_db_loc'] = download_and_process_dbcan(dbcan_loc, temporary, dbcan_release=dbcan_version,
-                                                            verbose=verbose)
-    print('%s: dbCAN database processed' % str(datetime.now() - start_time))
     output_dbs['viral_db_loc'] = download_and_process_viral_refseq(viral_loc, temporary, threads=threads,
                                                                    verbose=verbose)
     print('%s: RefSeq viral database processed' % str(datetime.now() - start_time))
@@ -478,12 +329,8 @@ def prepare_databases(output_dir, kegg_loc=None, gene_ko_link_loc=None, kofam_hm
     print('%s: KOfam ko list processed' % str(datetime.now() - start_time))
 
     # get pfam, dbcan and vogdb descriptions
-    if pfam_hmm_dat is None:
-        pfam_hmm_dat = download_pfam_descriptions(output_dir, verbose=verbose)
     output_dbs['pfam_hmm_dat'] = pfam_hmm_dat
     print('%s: PFAM hmm dat processed' % str(datetime.now() - start_time))
-    if dbcan_fam_activities is None:
-        dbcan_fam_activities = download_dbcan_descriptions(output_dir, dbcan_date, verbose=verbose)
     output_dbs['dbcan_fam_activities'] = dbcan_fam_activities
     print('%s: dbCAN fam activities processed' % str(datetime.now() - start_time))
     if vog_annotations is None:
@@ -523,37 +370,16 @@ def prepare_databases(output_dir, kegg_loc=None, gene_ko_link_loc=None, kofam_hm
 
     output_dbs['description_db_loc'] = path.realpath(path.join(output_dir, 'description_db.sqlite'))
 
-    set_database_paths(**output_dbs, use_current_locs=False, update_description_db=True, start_time=start_time)
+    db_handler = DatabaseHandler()
+    db_handler.populate_description_db(output_dbs['description_db_loc'], update_config=False)
+    db_handler.set_database_paths(**output_dbs)
+    print('%s: DRAM description database populated' % str(datetime.now() - start_time))
 
     if not keep_database_files:
         rmtree(temporary)
     print('%s: Database preparation completed' % str(datetime.now() - start_time))
 
 
-def print_database_locations(db_locs=None):
-    if db_locs is None:
-        db_locs = get_database_locs()
-
-    print('KEGG db: %s' % db_locs.get('kegg'))
-    print('KOfam db: %s' % db_locs.get('kofam'))
-    print('KOfam KO list: %s' % db_locs.get('kofam_ko_list'))
-    print('UniRef db: %s' % db_locs.get('uniref'))
-    print('Pfam db: %s' % db_locs.get('pfam'))
-    print('Pfam hmm dat: %s' % db_locs.get('pfam_hmm_dat'))
-    print('dbCAN db: %s' % db_locs.get('dbcan'))
-    print('dbCAN family activities: %s' % db_locs.get('dbcan_fam_activities'))
-    print('RefSeq Viral db: %s' % db_locs.get('viral'))
-    print('MEROPS peptidase db: %s' % db_locs.get('peptidase'))
-    print('VOGDB db: %s' % db_locs.get('vogdb'))
-    print('VOG annotations: %s' % db_locs.get('vog_annotations'))
-    print('Description db: %s' % db_locs.get('description_db'))
-    print('Genome summary form: %s' % db_locs.get('genome_summary_form'))
-    print('Module step form: %s' % db_locs.get('module_step_form'))
-    print('ETC module database: %s' % db_locs.get('etc_module_database'))
-    print('Function heatmap form: %s' % db_locs.get('function_heatmap_form'))
-    print('AMG database: %s' % db_locs.get('amg_database'))
-
-
 def update_dram_forms(output_dir, branch='master'):
     if not path.isdir(output_dir):
         mkdir(output_dir)
@@ -564,15 +390,5 @@ def update_dram_forms(output_dir, branch='master'):
     form_locs['etc_module_database_loc'] = download_and_process_etc_module_database(output_dir, branch)
     form_locs['function_heatmap_form_loc'] = download_and_process_function_heatmap_form(output_dir, branch)
     form_locs['amg_database_loc'] = download_and_process_amg_database(output_dir, branch)
-    set_database_paths(**form_locs, update_description_db=False)
-
-
-def export_config(output_file=None):
-    if output_file is None:
-        print(open(get_config_loc()).read())
-    else:
-        copy2(get_config_loc(), output_file)
-
-
-def import_config(config_loc):
-    copy2(config_loc, get_config_loc())
+    db_handler = DatabaseHandler()
+    db_handler.set_database_paths(**form_locs)
